//
//  i2c_lcd.h
//
//  I2C interface type Character LCD module Control
//  Copyright (C) 2009 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for I2C interface LCD module.
//
//	Target LCD module
//		LCD controller: ST7032i
//		LCD module    : Low voltage I2C LCD module SB1602B / Strowbery Linux Co.,Ltd.
//
//  Device: CY8C29466/27443  IDE: PSoC Designer 5 Service Pack 6(Build 1127)
//  Device: MSP430F2013      IDE: IAR Embedded Workbench for MSP430 V4.11B
//  Device: PIC18F14K50      IDE: MPLAB IDE 8.36, C Compiler for PIC18 MCUs v3.33
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2009.06.28 v0.0  First cording
//    2009.07.02 v0.0  Rewrite & 1st release
//    2009.07.05 v0.0  Compile error Bug fix
//    2009.07.06 v0.0  Startup wait 40ms->100ms
//    2009.08.02 v0.1  HI-TECH C -> ImageCraft C Compiler
//    2009.08.13 v0.1  Independ I2Cm and CPU Clock
//    2009.09.01 v0.2  Support "IAR EW MSP430" (Need TexasInstruments SLAA368368A Library)
//    2009.09.13 v0.3  Support "MPLAB+C18" and code reduction
//    2009.12.12 v0.3  Variable type BYTE,CHAR,INT -> uchar,char,int
//    2009.12.22 v0.3  External delay
//    2009.12.22 v0.3  Change API Name, I2C_LCD->i2c_lcd
//    2009.12.22 v0.3  Reset update history
//
//  Device: CY8C29466     IDE: PSoC Designer 5 Service Pack 6(Build 1127)
//  Device: MSP430F2013   IDE: IAR Embedded Workbench for MSP430 V4.11B
//  Device: PIC18F14K50   IDE: MPLAB IDE 8.36, C Compiler for PIC18 MCUs v3.33
//

#include "delay.h"						// Private
#include "i2c_lcd.h"					// Private

//
// Initial command & save registers
//
static unsigned char i2c_lcd_reg_func = 
            I2C_LCD_INST_FUNC
       |    I2C_LCD_DL_8BIT
//     |    I2C_LCD_DL_4BIT
       |    I2C_LCD_N_2LINE
//     |    I2C_LCD_N_1LINE
//     |    I2C_LCD_DH_DOUBLE
       |    I2C_LCD_DH_NORMAL
//     |    I2C_LCD_IS_EXTEND
//     |    I2C_LCD_IS_NORMAL
       ;
static unsigned char i2c_lcd_reg_bias = 
            I2C_LCD_INST_BIAS
//     |    I2C_LCD_BS_4
       |    I2C_LCD_BS_5
//     |    I2C_LCD_OSC_122HZ
//     |    I2C_LCD_OSC_131HZ 
//     |    I2C_LCD_OSC_144HZ
//     |    I2C_LCD_OSC_161HZ
       |    I2C_LCD_OSC_183HZ
//     |    I2C_LCD_OSC_221HZ
//     |    I2C_LCD_OSC_274HZ
//     |    I2C_LCD_OSC_347HZ 
//     |    I2C_LCD_OSC_120HZ
//     |    I2C_LCD_OSC_133HZ
//     |    I2C_LCD_OSC_149HZ
//     |    I2C_LCD_OSC_167HZ
//     |    I2C_LCD_OSC_192HZ
//     |    I2C_LCD_OSC_227HZ
//     |    I2C_LCD_OSC_277HZ
       ;
static unsigned char i2c_lcd_reg_cont = 
          I2C_LCD_INST_CONT
     |    (I2C_LCD_C_LOW & I2C_LCD_INIT_CONT)
     ;
static unsigned char i2c_lcd_reg_pow = 
            I2C_LCD_INST_POW
       |    I2C_LCD_I_ON                // icon
//     |    I2C_LCD_I_OFF 
       |    I2C_LCD_B_ON                // booster
//     |    I2C_LCD_B_OFF 
       |    (I2C_LCD_C_HIGH & (I2C_LCD_INIT_CONT >> 4))
       ;
static unsigned char i2c_lcd_reg_foll = 
            I2C_LCD_INST_FOLL
       |    I2C_LCD_F_ON                // switch follwer circuit
//     |    I2C_LCD_F_OFF 
//     |    I2C_LCD_RAB_1               // x1.0
//     |    I2C_LCD_RAB_1_25            // x1.25
//     |    I2C_LCD_RAB_1_5             // x1.5
       |    I2C_LCD_RAB_1_8             // x1.8
//     |    I2C_LCD_RAB_2               // x2.0
//     |    I2C_LCD_RAB_2_5             // x2.5
//     |    I2C_LCD_RAB_3               // x3.0
//     |    I2C_LCD_RAB_3_75            // x3.75
    ;
static unsigned char i2c_lcd_reg_disp = 
            I2C_LCD_INST_DISP
       |    I2C_LCD_DSP_ON
       |    I2C_LCD_DSP_OFF
//     |    I2C_LCD_CSL_ON
       |    I2C_LCD_CSL_OFF
//     |    I2C_LCD_BLK_ON
//     |    I2C_LCD_BLK_OFF
       ;

//
// Internal function
//
char i2c_lcd_itoc(int i) {
	return((i <= 9) ? (i+'0') : (i+'a'-10));
}

//
// API
//
void i2c_lcd_start(void) {
    I2C_LCD_START();
}

void i2c_lcd_stop(void) {
    I2C_LCD_STOP();
}

void i2c_lcd_sendCmdData(unsigned char cmd, char data){
    unsigned char I2C_LCD_CmdData[2];

    I2C_LCD_CmdData[0] = cmd;           // I2C_LCD_CB_xxx
    I2C_LCD_CmdData[1] = data;          // Data byte
    I2C_LCD_WRITE(
        I2C_LCD_ADDR,
        I2C_LCD_CmdData,
        sizeof(I2C_LCD_CmdData));
    i2c_lcd_delay50u();                 // 26.3us~

    return;
}

void i2c_lcd_init(void) {
    i2c_lcd_delay1mTimes(100);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_EXTEND);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_EXTEND);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_bias);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_cont);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_pow);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_foll);
    i2c_lcd_delay1mTimes(200);          // 200ms
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_disp);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_NORMAL);

    return;
}

// Override stdio -> usage: putchar(),puts(),printf()
//void  putch(char c){
//    i2c_lcd_sendCmdData(I2C_LCD_CB_DATA, c);
//    return;
//}

void i2c_lcd_prString(char *s) {
    for(; *s != '\0'; s++) {
        i2c_lcd_sendCmdData(I2C_LCD_CB_DATA, *s);
    }
    return;
}

void i2c_lcd_prCString(const char *s) {
    for(; *s != '\0'; s++) {
        i2c_lcd_sendCmdData(I2C_LCD_CB_DATA, *s);
    }
    return;
}

void i2c_lcd_prHexByte(unsigned char b) {
    char buf[1];

    buf[0] = i2c_lcd_itoc(b >> 4);
    i2c_lcd_sendCmdData(I2C_LCD_CB_DATA, buf[0]);
    buf[0] = i2c_lcd_itoc(b & 0xf);
    i2c_lcd_sendCmdData(I2C_LCD_CB_DATA, buf[0]);
    return;
}

void i2c_lcd_prHexInt(int i) {
    i2c_lcd_prHexByte(i >> 8);
    i2c_lcd_prHexByte(i & 0xff);
    return;
}

void  i2c_lcd_position(unsigned char bRow, unsigned char bCol){
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, I2C_LCD_INST_DADR | (bRow << 6) | (bCol));
    return;
}

void  i2c_lcd_contrast(unsigned char bCont){
    bCont &= 0x3f;
    i2c_lcd_reg_cont = (i2c_lcd_reg_cont & ~I2C_LCD_C_LOW ) | (I2C_LCD_C_LOW & bCont);
    i2c_lcd_reg_pow  = (i2c_lcd_reg_pow  & ~I2C_LCD_C_HIGH) | (bCont >> 4);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_EXTEND);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_cont);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_pow);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_NORMAL);
    return;
}

void  i2c_lcd_icon(unsigned char bAddr, unsigned char bFlag){
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_EXTEND);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, I2C_LCD_INST_ICON | (bAddr & I2C_LCD_ICON_ADDR));
    i2c_lcd_sendCmdData(I2C_LCD_CB_DATA, bFlag);
    i2c_lcd_sendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_NORMAL);
    return;
}
